"""This package gathers various supplementary features of SNAKES:

  * package `snakes.utils.abcd` features a compiler for the ABCD
    language, which is also available as a command-line tool
  * package `snakes.utils.ctlstar` is a compiler for CTL* formula
  * module `snakes.utils.apidoc` is used to extract SNAKES' API
    documentation and generate Markdown pages from it

"""
